# CTA  Notebook

This source code automates and replicates the computations presented in the motivating example from Section II of our paper, "A Correlation-Tolerant Analysis of the Deadline-Failure Probability of Dependent Tasks," which has been accepted for presentation at the RTSS '23 conference.

## Python Dependencies

All Python dependencies can be easily installed with Python's `pip3` package manager:

```terminal
pip3 install -r requirements.txt
```

## How to Compute the Results Presented in the Motivating Example?

### Step 1: Execute the Notebook

To execute all the cells in the notebook and save the output, run the following command in your terminal:

```terminal
jupyter nbconvert --to notebook --execute motivating_example_notebook.ipynb
```

This will execute all the code cells and retain the output within the notebook file (`motivating_example_notebook.nbconvert.ipynb`). The `--to notebook` flag specifies that the output format is a Jupyter Notebook, and `--execute` indicates that the code cells should be executed.

### Step 2: Convert to PDF

To convert the executed notebook to a PDF file, use the following command:

```terminal
jupyter nbconvert --to pdf motivating_example_notebook.ipynb
```

The `--to pdf` flag indicates that the output should be a PDF file. The notebook and its outputs will be converted into a PDF document named `motivating_example_notebook.pdf`.

---

## Alternative Output Formats

You can also convert the notebook into alternative formats by using the following commands:

### Convert to HTML

```terminal
jupyter nbconvert --to html motivating_example_notebook.ipynb
```

### Convert to Markdown

```terminal
jupyter nbconvert --to markdown motivating_example_notebook.ipynb
```

## Repository structure

This section outlines the repository's structure, detailing the functions of its various modules.

- `motivating_example_notebook.ipynb`: This is the primary Jupyter Notebook file used for recalculating the motivating example from the CTA paper.
- `probability_operations/`: This module contains essential operations related to probability.
- `joint_execution_abstraction/`: This module is responsible for calculating various attributes of the joint execution-time distribution for two tasks.
- `requirements.txt`: This file lists the project's dependencies, which can be installed using pip.
- `README.md`: This document provides a general overview of the project and includes instructions for setup and usage.
- `pre_generated_outputs`: This directory contains the pre-generated notebook (.ipynb) and PDF output files, which can serve as a fallback in the event that future dependencies disrupt the source code and/or instructions.

